import logging
import threading
from pathlib import Path
from threading import Thread
import signal

import vtk
from axipy import Table
from vtkmodules.vtkCommonCore import vtkCommand
import xml.etree.ElementTree as ET




class ProgressMonitor:
    """A standard class for monitoring the progress of a VTK algorithm.

    This must be use in a ``with`` context and it will block keyboard
    interrupts from happening until the exit event as interrupts will crash
    the kernel if the VTK algorithm is still executing.

    Parameters
    ----------
    algorithm
        VTK algorithm or filter.

    message : str, default: ""
        Message to display in the progress bar.

    scaling : float, optional
        Unused keyword argument.

    """

    def __init__(self, algorithm,call_back):
        """Initialize observer."""
        '''
        try:
            from tqdm import tqdm  # noqa
        except ImportError:
            raise ImportError("Please install `tqdm` to monitor algorithms.")
        '''
        self.event_type = vtkCommand.ProgressEvent

        self.algorithm = algorithm

        self._interrupt_signal_received = False
        self.__call_back_function=call_back

    def handler(self, sig, frame):
        """Pass signal to custom interrupt handler."""
        self._interrupt_signal_received = (sig, frame)
        logging.debug('SIGINT received. Delaying KeyboardInterrupt until VTK algorithm finishes.')

    def __call__(self, obj, event, *args):
        """Call progress update callback.

        On an event occurrence, this function executes.
        """
        if self._interrupt_signal_received:
            obj.AbortExecuteOn()
        else:
            progress = obj.GetProgress()
            isOk=self.__call_back_function(progress)
            if isOk==0:
                obj.AbortExecuteOn()


    def __enter__(self):
        """Enter event for ``with`` context."""

        self.algorithm.AddObserver(self.event_type, self)
        return

    def __exit__(self, type, value, traceback):
        """Exit event for ``with`` context."""

        self.algorithm.RemoveObservers(self.event_type)

def _update_alg(alg, call_back):
    """Update an algorithm with or without a progress bar."""

    with ProgressMonitor(alg, call_back):
            alg.Update()
def _writer_alg(alg, call_back):
    """Update an algorithm with or without a progress bar."""

    with ProgressMonitor(alg, call_back):
            alg.Write()
def writeAxiStl(path_stl,cs_mi,bound):
    path_dest = Path(path_stl)
    path_out=path_dest.parent
    path_out=path_out.joinpath(path_dest.stem+".axistl")
    rel_path = str(path_dest.relative_to(path_out.parent))
    root_axiStl = ET.Element('axiStl')
    path_file = ET.SubElement(root_axiStl, 'file')
    path_file.text = rel_path
    path_cs_mi = ET.SubElement(root_axiStl, 'cs_mi')
    path_cs_mi.text =cs_mi
    teg_bound = ET.SubElement(root_axiStl, 'bound')
    teg_bound_xmin= ET.SubElement(teg_bound, 'xmin')
    teg_bound_xmin.text=str(bound[0]).replace(',','.')
    teg_bound_ymin = ET.SubElement(teg_bound, 'ymin')
    teg_bound_ymin.text = str(bound[2]).replace(',','.')
    teg_bound_xmax = ET.SubElement(teg_bound, 'xmax')
    teg_bound_xmax.text = str(bound[1]).replace(',','.')
    teg_bound_ymax = ET.SubElement(teg_bound, 'ymax')
    teg_bound_ymax.text = str(bound[3]).replace(',','.')
    teg_bound_zmin = ET.SubElement(teg_bound, 'zmin')
    teg_bound_zmin.text = str(bound[4]).replace(',', '.')
    teg_bound_zmax = ET.SubElement(teg_bound, 'zmax')
    teg_bound_zmax.text = str(bound[5]).replace(',', '.')
    #xml_str = ET.tostring(root_axiStl, encoding="utf-8", method="xml")
    tree = ET.ElementTree(root_axiStl)
    with open(str(path_out), 'wb') as f:
        tree.write(f, encoding='utf-8', xml_declaration=True)
class AxiStl:
    def read(self,path_file):
        self.__path_source=path_file
        self.__tree = ET.parse(path_file)
        self.__root =self.__tree.getroot()
        '''
        tag_file_stl=self.__tree.find('file')
        self.__path_stl=tag_file_stl.text
        self.__abs_path_stl=str(Path(path_file).parent.joinpath(self.__path_stl))
        tag_cs_mi_stl = self.__tree.find('cs_mi')
        self.__cs_mi=tag_cs_mi_stl.text
        tag_xmin=self.__tree.find('bound/xmin')
        self.__xmin=float(tag_xmin.text)
        '''
        self.__path_stl=self.__getTextTag('file')
        self.__abs_path_stl = str(Path(path_file).parent.joinpath(self.__path_stl))
        self.__cs_mi = self.__getTextTag('cs_mi')
        self.__xmin=float(self.__getTextTag('bound/xmin'))
        self.__xmax = float(self.__getTextTag('bound/xmax'))
        self.__ymin = float(self.__getTextTag('bound/ymin'))
        self.__ymax = float(self.__getTextTag('bound/ymax'))
        self.__zmin = float(self.__getTextTag('bound/zmin'))
        self.__zmax = float(self.__getTextTag('bound/zmax'))
        jkl=0
    def __getTextTag(self,name_tag):
        tag_find=self.__tree.find(name_tag)
        if tag_find is None:
            return None
        return tag_find.text
    @property
    def StlFile(self):
        return self.__abs_path_stl
    @property
    def CsMiStr(self):
        return self.__cs_mi
    @property
    def Xmin(self):
        return self.__xmin
    @property
    def Ymin(self):
        return self.__ymin
    @property
    def Xmax(self):
        return self.__xmax
    @property
    def Ymax(self):
        return self.__ymax
    @property
    def Zmin(self):
        return self.__zmin
    @property
    def Zmax(self):
        return self.__zmax
def newZBound(zmin,zmax):
    width=zmax-zmin
    new_zmin=zmin-abs(width/100)
    new_zmax=zmax+abs(width/100)
    return new_zmin,new_zmax
