from multiprocessing.pool import ThreadPool

import numpy
from axipy import Polygon, MultiPolygon, CoordSystem
from osgeo import gdal, osr, ogr

def numpyFloat(value_numpy):
    ret_value=None
    try:
        ret_value=float(value_numpy)
    except:
        pass
    return ret_value

class CalcZoneStatistics:
    def __init__(self,path_source_raster,idBand=1):
        self.__path_source=path_source_raster
        self.__band_id=idBand
        self.__ds=gdal.Open(path_source_raster)
        self.__prepareRaster()
    def __prepareRaster(self):
        transform = self.__ds.GetGeoTransform()
        self.__xOrigin = transform[0]
        self.__yOrigin = transform[3]
        self.__pixelWidth = transform[1]
        self.__pixelHeight = transform[5]
        self.__gdal_targetSR = self.__ds.GetProjectionRef()
        self.__srs = osr.SpatialReference()
        self.__srs.ImportFromWkt(self.__gdal_targetSR)
        self.__cs_axi=CoordSystem.from_wkt(self.__gdal_targetSR)
        self.__memory_driver_ogr=ogr.GetDriverByName('MEMORY')
        self.__banddataraster = self.__ds.GetRasterBand(self.__band_id)
        self.__memory_ds=None
        self.__memory_lyr=None
    @property
    def ds(self):
        return self.__ds
    def calcStatistics(self,ft_data):
        geometry_source=ft_data['+geometry']
        #if not (isinstance(geometry_source,Polygon) or isinstance(geometry_source,MultiPolygon)):
        #    return None
        geometry=geometry_source
        if not(geometry.coordsystem==self.__cs_axi):
            geometry=geometry_source.reproject(self.__cs_axi)
        bound=geometry.bounds
        xmin=bound.xmin
        ymin=bound.ymin
        xmax=bound.xmax
        ymax=bound.ymax
        # Specify offset and rows and columns to read
        xoff = int((xmin - self.__xOrigin) / self.__pixelWidth)
        yoff = int((self.__yOrigin - ymax) / self.__pixelHeight)
        #col = int((xmin - self.__xOrigin) / self.__pixelWidth)
        #row = int((ymax - self.__yOrigin) / self.__pixelHeight)
        yoff = int((ymax-self.__yOrigin) / self.__pixelHeight)
        xcount = int((xmax - xmin) / self.__pixelWidth) + 1
        ycount = int((ymax - ymin) / (-self.__pixelHeight)) + 1
        dataraster_temp = self.__banddataraster.ReadAsArray(xoff, yoff, xcount, ycount)
        if dataraster_temp is None:
            return ft_data
        dataraster = self.__banddataraster.ReadAsArray(xoff, yoff, xcount, ycount).astype(float)
        if xcount * ycount == 1:

            out_result = {}

            out_result['value_average'] = numpyFloat(numpy.average(dataraster))
            out_result['value_mean'] = numpyFloat(numpy.mean(dataraster))
            out_result['value_median'] = numpyFloat(numpy.median(dataraster))
            out_result['value_min'] = numpyFloat(numpy.min(dataraster))
            out_result['value_max'] = numpyFloat(numpy.max(dataraster))
            out_result['value_count'] = numpy.size(dataraster)
            dataraster = None
            dataraster_temp=None
            return out_result
        # Create memory target raster
        target_ds = gdal.GetDriverByName('MEM').Create('', xcount, ycount, 1, gdal.GDT_Byte)
        target_ds.SetGeoTransform((
            xmin, self.__pixelWidth, 0,
            ymax, 0, self.__pixelHeight,
        ))
        # Create for target raster the same projection as for the value raster
        raster_srs = osr.SpatialReference()
        raster_srs.ImportFromWkt(self.__ds.GetProjectionRef())
        target_ds.SetProjection(raster_srs.ExportToWkt())
        meme_ds,meme_layer=self.__createMemoryLayer(geometry)
        # Rasterize zone polygon to raster
        gdal.RasterizeLayer(target_ds, [1], meme_layer, burn_values=[1])
        meme_layer=None
        meme_ds=None

        bandmask = target_ds.GetRasterBand(1)
        datamask = bandmask.ReadAsArray(0, 0, xcount, ycount).astype(float)

        # Mask zone of raster
        zoneraster = numpy.ma.masked_array(dataraster, numpy.logical_not(datamask))
        target_ds=None
        bandmask=None
        banddataraster=None
        out_result={}

        out_result['value_average']=numpyFloat(numpy.average(zoneraster))
        out_result['value_mean'] = numpyFloat(numpy.mean(zoneraster))
        out_result['value_median'] = numpyFloat(numpy.median(zoneraster))
        out_result['value_min'] = numpyFloat(numpy.min(zoneraster))
        out_result['value_max'] = numpyFloat(numpy.max(zoneraster))
        out_result['value_count'] = numpy.size(zoneraster)
        return   {**ft_data, **out_result}
    def calcStatisticsExt(self,ft_data):
        geometry_source=ft_data['+geometry']
        #if not (isinstance(geometry_source,Polygon) or isinstance(geometry_source,MultiPolygon)):
        #    return None
        geometry=geometry_source
        if not(geometry.coordsystem==self.__cs_axi):
            geometry=geometry_source.reproject(self.__cs_axi)
        bound=geometry.bounds
        xmin=bound.xmin
        ymin=bound.ymin
        xmax=bound.xmax
        ymax=bound.ymax
        # Specify offset and rows and columns to read
        xoff = int((xmin - self.__xOrigin) / self.__pixelWidth)
        yoff = int((self.__yOrigin - ymax) / self.__pixelHeight)
        #col = int((xmin - self.__xOrigin) / self.__pixelWidth)
        #row = int((ymax - self.__yOrigin) / self.__pixelHeight)
        yoff = int((ymax-self.__yOrigin) / self.__pixelHeight)
        xcount = int((xmax - xmin) / self.__pixelWidth) + 1
        ycount = int((ymax - ymin) / (-self.__pixelHeight)) + 1
        dataraster_temp = self.__banddataraster.ReadAsArray(xoff, yoff, xcount, ycount)
        if dataraster_temp is None:
            return ft_data
        # Create memory target raster
        target_ds = gdal.GetDriverByName('MEM').Create('', xcount, ycount, 1, gdal.GDT_Byte)
        target_ds.SetGeoTransform((
            xmin, self.__pixelWidth, 0,
            ymax, 0, self.__pixelHeight,
        ))
        # Create for target raster the same projection as for the value raster
        raster_srs = osr.SpatialReference()
        raster_srs.ImportFromWkt(self.__ds.GetProjectionRef())
        target_ds.SetProjection(raster_srs.ExportToWkt())
        meme_ds,meme_layer=self.__createMemoryLayer(geometry)
        # Rasterize zone polygon to raster
        gdal.RasterizeLayer(target_ds, [1], meme_layer, burn_values=[1])
        meme_layer=None
        meme_ds=None
        # Read raster as arrays
        #banddataraster = self.__ds.GetRasterBand(1)


        dataraster =dataraster_temp.astype(float)
        if xcount*ycount==1:
            #banddataraster=None
            out_result = {}

            out_result['value_average'] = numpyFloat(numpy.average(dataraster))
            out_result['value_mean'] = numpyFloat(numpy.mean(dataraster))
            out_result['value_median'] = numpyFloat(numpy.median(dataraster))
            out_result['value_min'] = numpyFloat(numpy.min(dataraster))
            out_result['value_max'] = numpyFloat(numpy.max(dataraster))
            out_result['value_count'] = numpy.size(dataraster)
            dataraster=None
            return {**ft_data, **out_result}
        bandmask = target_ds.GetRasterBand(1)
        datamask = bandmask.ReadAsArray(0, 0, xcount, ycount).astype(float)

        # Mask zone of raster
        zoneraster = numpy.ma.masked_array(dataraster, numpy.logical_not(datamask))
        target_ds=None
        bandmask=None
        banddataraster=None
        out_result={}

        out_result['value_average']=numpyFloat(numpy.average(zoneraster))
        out_result['value_mean'] = numpyFloat(numpy.mean(zoneraster))
        out_result['value_median'] = numpyFloat(numpy.median(zoneraster))
        out_result['value_min'] = numpyFloat(numpy.min(zoneraster))
        out_result['value_max'] = numpyFloat(numpy.max(zoneraster))
        out_result['value_count'] = numpy.size(zoneraster)
        return {**ft_data, **out_result}
    def __createMemoryLayer(self,geometry):
        memory_lyr = None
        memory_ds = None

        memory_ds=self.__memory_driver_ogr.CreateDataSource('memData')
        memory_lyr=memory_ds.CreateLayer("meme_polygon", self.__srs, ogr.wkbPolygon)
        feature = ogr.Feature(memory_lyr.GetLayerDefn())
        ogr_geo=geometry=ogr.CreateGeometryFromWkb(geometry.to_wkb())
        feature.SetGeometry(ogr_geo)
        # Create the feature in the layer (shapefile)
        memory_lyr.CreateFeature(feature)
        ogr_geo=None
        feature=None
        return memory_ds,memory_lyr
    def calcStatisticsListPool(self,list_data_ft,size_pool=8):
        p = ThreadPool(size_pool)
        result = p.map(self.calcStatisticsExt, list_data_ft)
        p.close()
        p.join()
        p.close()
        return result
    def calcStatisticsList(self,list_data_ft):
        list_ft_out=[]
        for item in list_data_ft:
            data_statistics=self.calcStatistics(item)
            #if data_statistics is None:
            #    list_ft_out.append(item)
            #    continue
            #list_ft_out.append({**item, **data_statistics})
            list_ft_out.append(data_statistics)
        return list_ft_out
    def closeDs(self):
        self.__banddataraster=None
        self.__memory_lyr = None
        self.__memory_ds = None
        self.__ds=None